@@ -32,8 +32,8 @@ import logging
 import os
 import re
 import shutil
-from tarfile import TarFile
 from abc import ABCMeta, abstractmethod
+from contextlib import closing
 from functools import partial
 
 import dateutil.parser
@@ -52,6 +52,7 @@ from barman.exceptions import (
     PostgresConnectionError,
     PostgresIsInRecovery,
     SshCommandException,
+    FileNotFoundException,
 )
 from barman.fs import UnixLocalCommand, UnixRemoteCommand
 from barman.infofile import BackupInfo
@@ -269,9 +270,8 @@ class PostgresBackupExecutor(BackupExecutor):
         # Forbid tablespace_bandwidth_limit option.
         # It works only with rsync based backups.
         if self.config.tablespace_bandwidth_limit:
-            self.server.config.disabled = True
             # Report the error in the configuration errors message list
-            self.server.config.msg_list.append(
+            self.server.config.update_msg_list_and_disable_server(
                 "tablespace_bandwidth_limit option is not supported by "
                 "postgres backup_method"
             )
@@ -279,46 +279,76 @@ class PostgresBackupExecutor(BackupExecutor):
         # Forbid reuse_backup option.
         # It works only with rsync based backups.
         if self.config.reuse_backup in ("copy", "link"):
-            self.server.config.disabled = True
             # Report the error in the configuration errors message list
-            self.server.config.msg_list.append(
+            self.server.config.update_msg_list_and_disable_server(
                 "reuse_backup option is not supported by postgres backup_method"
             )
 
         # Forbid network_compression option.
         # It works only with rsync based backups.
         if self.config.network_compression:
-            self.server.config.disabled = True
             # Report the error in the configuration errors message list
-            self.server.config.msg_list.append(
+            self.server.config.update_msg_list_and_disable_server(
                 "network_compression option is not supported by "
                 "postgres backup_method"
             )
 
         remote_status = None
-        # bandwidth_limit option is supported by pg_basebackup executable
-        # starting from Postgres 9.4
-        if self.server.config.bandwidth_limit:
+        if self.server.config.bandwidth_limit or self.backup_compression:
             # This method is invoked too early to have a working streaming
             # connection. So we avoid caching the result by directly
             # invoking fetch_remote_status() instead of get_remote_status()
-            remote_status = self.fetch_remote_status()
+            # We make the call within a closing context manager so we do not
+            # leave any dangling PostgreSQL connections in cases where no
+            # further server interaction is required.
+            with closing(self.server):
+                remote_status = self.fetch_remote_status()
+
+        # bandwidth_limit option is supported by pg_basebackup executable
+        # starting from Postgres 9.4
+        if (
+            self.server.config.bandwidth_limit
+            and remote_status["pg_basebackup_bwlimit"] is False
+        ):
             # If pg_basebackup is present and it doesn't support bwlimit
             # disable the server.
-            if remote_status["pg_basebackup_bwlimit"] is False:
-                self.server.config.disabled = True
-                # Report the error in the configuration errors message list
-                self.server.config.msg_list.append(
-                    "bandwidth_limit option is not supported by "
-                    "pg_basebackup version (current: %s, required: 9.4)"
-                    % remote_status["pg_basebackup_version"]
-                )
+            # Report the error in the configuration errors message list
+            self.server.config.update_msg_list_and_disable_server(
+                "bandwidth_limit option is not supported by "
+                "pg_basebackup version (current: %s, required: 9.4)"
+                % remote_status["pg_basebackup_version"]
+            )
 
         # validate compression options
         if self.backup_compression:
-            if remote_status is None:
-                remote_status = self.fetch_remote_status()
-            self.backup_compression.validate(self.server, remote_status)
+            self._validate_compression(remote_status)
+
+    def _validate_compression(self, remote_status):
+        """
+        In charge of validating compression options
+        :param remote_status:
+        :return:
+        """
+        try:
+            issues = self.backup_compression.validate(
+                self.server.postgres.server_version, remote_status
+            )
+            if issues:
+                self.server.config.update_msg_list_and_disable_server(issues)
+        except PostgresConnectionError as exc:
+            # If we can't validate the compression settings due to a connection error
+            # it should not block whatever Barman is trying to do *unless* it is
+            # doing a backup, in which case the pre-backup check will catch the
+            # connection error and fail accordingly.
+            # This is important because if the server is unavailable Barman
+            # commands such as `recover` and `list-backups` must not break.
+            _logger.warning(
+                (
+                    "Could not validate compression due to a problem "
+                    "with the PostgreSQL connection: %s"
+                ),
+                exc,
+            )
 
     def backup(self, backup_info):
         """
@@ -584,6 +614,7 @@ class PostgresBackupExecutor(BackupExecutor):
             retry_sleep=self.config.basebackup_retry_sleep,
             retry_handler=partial(self._retry_handler, dest_dirs),
             compression=self.backup_compression,
+            err_handler=self._err_handler,
         )
 
         # Do the actual copy
@@ -648,6 +679,32 @@ class PostgresBackupExecutor(BackupExecutor):
         # Remove all the destination directories and reinit the backup
         self._prepare_backup_destination(dest_dirs)
 
+    def _err_handler(self, line):
+        """
+        Handler invoked during a backup when anything is sent to stderr.
+
+        Used to perform a WAL switch on a primary server if pg_basebackup
+        is running against a standby, otherwise just logs output at INFO
+        level.
+
+        :param str line: The error line to be handled.
+        """
+
+        # Always log the line, since this handler will have overridden the
+        # default command err_handler.
+        # Although this is used as a stderr handler, the pg_basebackup lines
+        # logged here are more appropriate at INFO level since they are just
+        # describing regular behaviour.
+        _logger.log(logging.INFO, "%s", line)
+        if (
+            self.server.config.primary_conninfo is not None
+            and "waiting for required WAL segments to be archived" in line
+        ):
+            # If pg_basebackup is waiting for WAL segments and primary_conninfo
+            # is configured then we are backing up a standby and must manually
+            # perform a WAL switch.
+            self.server.postgres.switch_wal()
+
     def _prepare_backup_destination(self, dest_dirs):
         """
         Prepare the destination of the backup, including tablespaces.
@@ -1118,8 +1175,7 @@ class RsyncBackupExecutor(FsBackupExecutor):
     def validate_configuration(self):
         # Verify that backup_compression is not set
         if self.server.config.backup_compression:
-            self.server.config.disabled = True
-            self.server.config.msg_list.append(
+            self.server.config.update_msg_list_and_disable_server(
                 "backup_compression option is not supported by rsync backup_method"
             )
 
@@ -1604,8 +1660,10 @@ class PostgresBackupStrategy(BackupStrategy):
 
         :param barman.infofile.LocalBackupInfo backup_info: backup information
         """
-        if self.backup_compression and self.backup_compression.format != "plain":
-            backup_info.set_attribute("compression", self.backup_compression.type)
+        if self.backup_compression and self.backup_compression.config.format != "plain":
+            backup_info.set_attribute(
+                "compression", self.backup_compression.config.type
+            )
 
         self._read_backup_label(backup_info)
         self._backup_info_from_backup_label(backup_info)
@@ -1634,13 +1692,10 @@ class PostgresBackupStrategy(BackupStrategy):
 
         :param barman.infofile.LocalBackupInfo backup_info: backup information
         """
-        basename = os.path.join(backup_info.get_data_directory(), "base.tar")
-        with self.backup_compression.open(basename) as f:
-            self.tar = TarFile.open(fileobj=f, mode="r|")
-            for member in self.tar:
-                if member.name == "backup_label":
-                    backup_label_fileobj = self.tar.extractfile(member)
-                    return backup_label_fileobj.read().decode("utf-8")
+        basename = os.path.join(backup_info.get_data_directory(), "base")
+        try:
+            return self.backup_compression.get_file_content("backup_label", basename)
+        except FileNotFoundException:
             raise BackupException(
                 "Could not find backup_label in %s"
                 % self.backup_compression.with_suffix(basename)
