@@ -92,7 +92,7 @@ def parse_arguments(args=None):
     parser, _, _ = create_argument_parser(
         description="This script can be used to download a backup "
         "previously made with barman-cloud-backup command."
-        "Currently AWS S3 and Azure Blob Storage are supported.",
+        "Currently AWS S3, Azure Blob Storage and Google Cloud Storage are supported.",
     )
     parser.add_argument("backup_id", help="the backup ID")
     parser.add_argument("recovery_dir", help="the path to a directory for recovery.")
@@ -165,7 +165,7 @@ class CloudBackupDownloader(object):
         # We must download and restore a bunch of .tar files that contain PGDATA
         # and each tablespace. First, we determine a target directory to extract
         # each tar file into and record these in copy_jobs. For each tablespace,
-        # the location may be overriden by `--tablespace name:/new/location` on
+        # the location may be overridden by `--tablespace name:/new/location` on
         # the command-line; and we must also add an entry to link_jobs to create
         # a symlink from $PGDATA/pg_tblspc/oid to the correct location after the
         # downloads.
