@@ -8,7 +8,7 @@
 #
 # See the help page for usage information.
 #
-# © Copyright EnterpriseDB UK Limited 2016-2021
+# © Copyright EnterpriseDB UK Limited 2016-2022
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 
 from __future__ import print_function
 
+import argparse
 import os
 import shutil
 import subprocess
@@ -34,11 +35,6 @@ import time
 import barman
 from barman.utils import force_str
 
-try:
-    import argparse
-except ImportError:
-    raise SystemExit("Missing required python module: argparse")
-
 DEFAULT_USER = "barman"
 DEFAULT_SPOOL_DIR = "/var/tmp/walrestore"
 
@@ -63,14 +59,17 @@ def main(args=None):
 
     # Check WAL destination is not a directory
     if os.path.isdir(config.wal_dest):
-        exit_with_error("WAL_DEST cannot be a directory: %s" % config.wal_dest)
+        exit_with_error(
+            "WAL_DEST cannot be a directory: %s" % config.wal_dest, status=3
+        )
 
     # Open the destination file
     try:
         dest_file = open(config.wal_dest, "wb")
     except EnvironmentError as e:
         exit_with_error(
-            "Cannot open '%s' (WAL_DEST) for writing: %s" % (config.wal_dest, e)
+            "Cannot open '%s' (WAL_DEST) for writing: %s" % (config.wal_dest, e),
+            status=3,
         )
         return  # never reached
 
@@ -104,9 +103,9 @@ def main(args=None):
     if ssh_process.returncode == 0:
         sys.exit(0)
 
-    # Report the exit code, remapping ssh failure code (255) to 3
+    # Report the exit code, remapping ssh failure code (255) to 2
     if ssh_process.returncode == 255:
-        exit_with_error("Connection problem with ssh", 3, sleep=config.sleep)
+        exit_with_error("Connection problem with ssh", 2, sleep=config.sleep)
     else:
         exit_with_error(
             "Remote 'barman get-wal' command has failed!",
@@ -181,8 +180,10 @@ def build_ssh_command(config, wal_name, peek=0):
     :param int peek: in
     :return list[str]: the ssh command as list of string
     """
-    ssh_command = [
-        "ssh",
+    ssh_command = ["ssh"]
+    if config.port is not None:
+        ssh_command += ["-p", config.port]
+    ssh_command += [
         "-q",  # quiet mode - suppress warnings
         "-T",  # disable pseudo-terminal allocation
         "%s@%s" % (config.user, config.barman_host),
@@ -317,6 +318,10 @@ def parse_arguments(args=None):
         help="The user used for the ssh connection to the Barman server. "
         "Defaults to '%(default)s'.",
     )
+    parser.add_argument(
+        "--port",
+        help="The port used for the ssh connection to the Barman server.",
+    )
     parser.add_argument(
         "-s",
         "--sleep",
@@ -390,17 +395,17 @@ def parse_arguments(args=None):
     parser.add_argument(
         "server_name",
         metavar="SERVER_NAME",
-        help="The server name configured in Barman " "from which WALs are taken.",
+        help="The server name configured in Barman from which WALs are taken.",
     )
     parser.add_argument(
         "wal_name",
         metavar="WAL_NAME",
-        help="The value of the '%%f' keyword " "(according to 'restore_command').",
+        help="The value of the '%%f' keyword (according to 'restore_command').",
     )
     parser.add_argument(
         "wal_dest",
         metavar="WAL_DEST",
-        help="The value of the '%%p' keyword " "(according to 'restore_command').",
+        help="The value of the '%%p' keyword (according to 'restore_command').",
     )
     return parser.parse_args(args=args)
 
