@@ -45,6 +45,7 @@ from barman.utils import (
     force_str,
     human_readable_timedelta,
     pretty_size,
+    range_fun,
     total_seconds,
     with_metaclass,
 )
@@ -635,7 +636,16 @@ class CloudInterface(with_metaclass(ABCMeta)):
         """
         pass
 
-    def __init__(self, url, jobs=2, tags=None):
+    @abstractproperty
+    def MAX_DELETE_BATCH_SIZE(self):
+        """
+        The maximum number of objects which can be deleted in a single batch.
+
+        :type: int
+        """
+        pass
+
+    def __init__(self, url, jobs=2, tags=None, delete_batch_size=None):
         """
         Base constructor
 
@@ -644,10 +654,22 @@ class CloudInterface(with_metaclass(ABCMeta)):
           uploading, defaults to 2.
         :param List[tuple] tags: List of tags as k,v tuples to be added to all
           uploaded objects
+        :param int|None delete_batch_size: the maximum number of objects to be
+          deleted in a single request
         """
         self.url = url
         self.tags = tags
 
+        # We use the maximum allowed batch size by default.
+        self.delete_batch_size = self.MAX_DELETE_BATCH_SIZE
+        if delete_batch_size is not None:
+            # If a specific batch size is requested we clamp it between 1 and the
+            # maximum allowed batch size.
+            self.delete_batch_size = max(
+                1,
+                min(delete_batch_size, self.MAX_DELETE_BATCH_SIZE),
+            )
+
         # The worker process and the shared queue are created only when
         # needed
         self.queue = None
@@ -1140,12 +1162,39 @@ class CloudInterface(with_metaclass(ABCMeta)):
         """
 
     @abstractmethod
+    def _delete_objects_batch(self, paths):
+        """
+        Delete a single batch of objects
+
+        :param List[str] paths:
+        """
+        if len(paths) > self.MAX_DELETE_BATCH_SIZE:
+            raise ValueError("Max batch size exceeded")
+
     def delete_objects(self, paths):
         """
         Delete the objects at the specified paths
 
+        Deletes the objects defined by the supplied list of paths in batches
+        specified by either batch_size or MAX_DELETE_BATCH_SIZE, whichever is
+        lowest.
+
         :param List[str] paths:
         """
+        errors = False
+        for i in range_fun(0, len(paths), self.delete_batch_size):
+            try:
+                self._delete_objects_batch(paths[i : i + self.delete_batch_size])
+            except CloudProviderError:
+                # Don't let one error stop us from trying to delete any remaining
+                # batches.
+                errors = True
+
+        if errors:
+            raise CloudProviderError(
+                "Error from cloud provider while deleting objects - "
+                "please check the command output."
+            )
 
 
 class CloudBackupUploader(with_metaclass(ABCMeta)):
@@ -1312,7 +1361,7 @@ class CloudBackupUploader(with_metaclass(ABCMeta)):
 
 class CloudBackupUploaderBarman(CloudBackupUploader):
     """
-    A cloud storage upload client for a pre-existing backup on the Barman server.
+    A cloud storage upload client for a preexisting backup on the Barman server.
     """
 
     def __init__(
@@ -1620,7 +1669,7 @@ class CloudBackupCatalog(KeepManagerMixinCloud):
 
     def __init__(self, cloud_interface, server_name):
         """
-        Object responsible for retrievin backup catalog from cloud storage
+        Object responsible for retrieving backup catalog from cloud storage
 
         :param CloudInterface cloud_interface: The interface to use to
           upload the backup
