@@ -49,6 +49,7 @@ def _make_s3_cloud_interface(config, cloud_interface_kwargs):
         {
             "profile_name": config.profile,
             "endpoint_url": config.endpoint_url,
+            "read_timeout": config.read_timeout,
         }
     )
     if "encryption" in config:
@@ -92,22 +93,40 @@ def _make_azure_cloud_interface(config, cloud_interface_kwargs):
     return AzureCloudInterface(**cloud_interface_kwargs)
 
 
+def _make_google_cloud_interface(config, cloud_interface_kwargs):
+    """
+    :param config: Not used yet
+    :param cloud_interface_kwargs: common parameters
+    :return: GoogleCloudInterface
+    """
+    from barman.cloud_providers.google_cloud_storage import GoogleCloudInterface
+
+    cloud_interface_kwargs["jobs"] = 1
+    return GoogleCloudInterface(**cloud_interface_kwargs)
+
+
 def get_cloud_interface(config):
     """
-    Create a CloudInterface for the specified cloud_provider
+    Factory function that creates CloudInterface for the specified cloud_provider
 
+    :param: argparse.Namespace config
     :returns: A CloudInterface for the specified cloud_provider
     :rtype: CloudInterface
     """
     cloud_interface_kwargs = {
         "url": config.source_url if "source_url" in config else config.destination_url
     }
-    _update_kwargs(cloud_interface_kwargs, config, ("jobs", "tags"))
+    _update_kwargs(
+        cloud_interface_kwargs, config, ("jobs", "tags", "delete_batch_size")
+    )
 
     if config.cloud_provider == "aws-s3":
         return _make_s3_cloud_interface(config, cloud_interface_kwargs)
     elif config.cloud_provider == "azure-blob-storage":
         return _make_azure_cloud_interface(config, cloud_interface_kwargs)
+    elif config.cloud_provider == "google-cloud-storage":
+
+        return _make_google_cloud_interface(config, cloud_interface_kwargs)
     else:
         raise CloudProviderUnsupported(
             "Unsupported cloud provider: %s" % config.cloud_provider
