@@ -21,7 +21,12 @@ import shutil
 from io import RawIOBase
 
 from barman.clients.cloud_compression import decompress_to_file
-from barman.cloud import CloudInterface, CloudProviderError, DecompressingStreamingIO
+from barman.cloud import (
+    CloudInterface,
+    CloudProviderError,
+    DecompressingStreamingIO,
+    DEFAULT_DELIMITER,
+)
 
 
 try:
@@ -34,6 +39,7 @@ except ImportError:
 
 try:
     import boto3
+    from botocore.config import Config
     from botocore.exceptions import ClientError, EndpointConnectionError
 except ImportError:
     raise SystemExit("Missing required python module: boto3")
@@ -67,6 +73,8 @@ class S3CloudInterface(CloudInterface):
     # MAX_ARCHIVE_SIZE - so we set a maximum of 1TB per file
     MAX_ARCHIVE_SIZE = 1 << 40
 
+    MAX_DELETE_BATCH_SIZE = 1000
+
     def __getstate__(self):
         state = self.__dict__.copy()
         # Remove boto3 client reference from the state as it cannot be pickled
@@ -88,6 +96,8 @@ class S3CloudInterface(CloudInterface):
         profile_name=None,
         endpoint_url=None,
         tags=None,
+        delete_batch_size=None,
+        read_timeout=None,
     ):
         """
         Create a new S3 interface given the S3 destination url and the profile
@@ -100,15 +110,21 @@ class S3CloudInterface(CloudInterface):
         :param str profile_name: Amazon auth profile identifier
         :param str endpoint_url: override default endpoint detection strategy
           with this one
+        :param int|None delete_batch_size: the maximum number of objects to be
+          deleted in a single request
+        :param int|None read_timeout: the time in seconds until a timeout is
+          raised when waiting to read from a connection
         """
         super(S3CloudInterface, self).__init__(
             url=url,
             jobs=jobs,
             tags=tags,
+            delete_batch_size=delete_batch_size,
         )
         self.profile_name = profile_name
         self.encryption = encryption
         self.endpoint_url = endpoint_url
+        self.read_timeout = read_timeout
 
         # Extract information from the destination URL
         parsed_url = urlparse(url)
@@ -126,8 +142,13 @@ class S3CloudInterface(CloudInterface):
         """
         Create a new session
         """
+        config_kwargs = {}
+        if self.read_timeout is not None:
+            config_kwargs["read_timeout"] = self.read_timeout
+        config = Config(**config_kwargs)
+
         session = boto3.Session(profile_name=self.profile_name)
-        self.s3 = session.resource("s3", endpoint_url=self.endpoint_url)
+        self.s3 = session.resource("s3", endpoint_url=self.endpoint_url, config=config)
 
     @property
     def _extra_upload_args(self):
@@ -135,7 +156,7 @@ class S3CloudInterface(CloudInterface):
         Return a dict containing ExtraArgs to be passed to certain boto3 calls
 
         Because some boto3 calls accept `ExtraArgs: {}` and others do not, we
-        return a nexted dict which can be expanded with `**` in the boto3 call.
+        return a nested dict which can be expanded with `**` in the boto3 call.
         """
         additional_args = {}
         if self.encryption:
@@ -203,7 +224,7 @@ class S3CloudInterface(CloudInterface):
             }
         self.s3.Bucket(self.bucket_name).create(**create_bucket_config)
 
-    def list_bucket(self, prefix="", delimiter="/"):
+    def list_bucket(self, prefix="", delimiter=DEFAULT_DELIMITER):
         """
         List bucket content in a directory manner
 
@@ -237,7 +258,7 @@ class S3CloudInterface(CloudInterface):
 
         :param str key: The S3 key to download
         :param str dest_path: Where to put the destination file
-        :param bool decompress: Whenever to decompress this file or not
+        :param str|None decompress: Compression scheme to use for decompression
         """
         # Open the remote file
         obj = self.s3.Object(self.bucket_name, key)
@@ -246,7 +267,7 @@ class S3CloudInterface(CloudInterface):
         # Write the dest file in binary mode
         with open(dest_path, "wb") as dest_file:
             # If the file is not compressed, just copy its content
-            if not decompress:
+            if decompress is None:
                 shutil.copyfileobj(remote_file, dest_file)
                 return
 
@@ -358,43 +379,25 @@ class S3CloudInterface(CloudInterface):
             Bucket=self.bucket_name, Key=key, UploadId=upload_metadata["UploadId"]
         )
 
-    def delete_objects(self, paths):
+    def _delete_objects_batch(self, paths):
         """
         Delete the objects at the specified paths
 
         :param List[str] paths:
         """
-        # Explicitly check if we are being asked to delete nothing at all and if
-        # so return without error.
-        if len(paths) == 0:
-            return
+        super(S3CloudInterface, self)._delete_objects_batch(paths)
 
-        # S3 bulk deletion is limited to batches of 1000 keys
-        batch_size = 1000
-        try:
-            # If xrange exists then we are on python 2 so we need to use it
-            range_fun = xrange
-        except NameError:
-            # Otherwise just use range
-            range_fun = range
-        errors = False
-        for i in range_fun(0, len(paths), batch_size):
-            resp = self.s3.meta.client.delete_objects(
-                Bucket=self.bucket_name,
-                Delete={
-                    "Objects": [{"Key": path} for path in paths[i : i + batch_size]],
-                    "Quiet": True,
-                },
-            )
-            if "Errors" in resp:
-                errors = True
-                for error_dict in resp["Errors"]:
-                    logging.error(
-                        'Deletion of object %s failed with error code: "%s", message: "%s"'
-                        % (error_dict["Key"], error_dict["Code"], error_dict["Message"])
-                    )
-        if errors:
-            raise CloudProviderError(
-                "Error from cloud provider while deleting objects - "
-                "please check the Barman logs"
-            )
+        resp = self.s3.meta.client.delete_objects(
+            Bucket=self.bucket_name,
+            Delete={
+                "Objects": [{"Key": path} for path in paths],
+                "Quiet": True,
+            },
+        )
+        if "Errors" in resp:
+            for error_dict in resp["Errors"]:
+                logging.error(
+                    'Deletion of object %s failed with error code: "%s", message: "%s"'
+                    % (error_dict["Key"], error_dict["Code"], error_dict["Message"])
+                )
+            raise CloudProviderError()
