@@ -22,7 +22,12 @@ import requests
 from io import BytesIO, RawIOBase, SEEK_END
 
 from barman.clients.cloud_compression import decompress_to_file
-from barman.cloud import CloudInterface, CloudProviderError, DecompressingStreamingIO
+from barman.cloud import (
+    CloudInterface,
+    CloudProviderError,
+    DecompressingStreamingIO,
+    DEFAULT_DELIMITER,
+)
 
 try:
     # Python 3.x
@@ -101,6 +106,8 @@ class AzureCloudInterface(CloudInterface):
     # MAX_ARCHIVE_SIZE - so we set a maximum of 1TB per file
     MAX_ARCHIVE_SIZE = 1 << 40
 
+    MAX_DELETE_BATCH_SIZE = 256
+
     # The size of each chunk in a single object upload when the size of the
     # object exceeds max_single_put_size. We default to 2MB in order to
     # allow the default max_concurrency of 8 to be achieved when uploading
@@ -128,21 +135,25 @@ class AzureCloudInterface(CloudInterface):
         encryption_scope=None,
         credential=None,
         tags=None,
+        delete_batch_size=None,
         max_block_size=DEFAULT_MAX_BLOCK_SIZE,
         max_concurrency=DEFAULT_MAX_CONCURRENCY,
         max_single_put_size=DEFAULT_MAX_SINGLE_PUT_SIZE,
     ):
         """
-        Create a new Azure Blob Storage interface given the supplied acccount url
+        Create a new Azure Blob Storage interface given the supplied account url
 
         :param str url: Full URL of the cloud destination/source
         :param int jobs: How many sub-processes to use for asynchronous
           uploading, defaults to 2.
+        :param int|None delete_batch_size: the maximum number of objects to be
+          deleted in a single request
         """
         super(AzureCloudInterface, self).__init__(
             url=url,
             jobs=jobs,
             tags=tags,
+            delete_batch_size=delete_batch_size,
         )
         self.encryption_scope = encryption_scope
         self.credential = credential
@@ -293,7 +304,7 @@ class AzureCloudInterface(CloudInterface):
                 for v in self._walk_blob_tree(child):
                     yield v
 
-    def list_bucket(self, prefix="", delimiter="/"):
+    def list_bucket(self, prefix="", delimiter=DEFAULT_DELIMITER):
         """
         List bucket content in a directory manner
 
@@ -317,7 +328,7 @@ class AzureCloudInterface(CloudInterface):
         """
         obj = self.container_client.download_blob(key)
         with open(dest_path, "wb") as dest_file:
-            if not decompress:
+            if decompress is None:
                 obj.download_to_stream(dest_file)
                 return
             blob = StreamingBlobIO(obj)
@@ -446,12 +457,14 @@ class AzureCloudInterface(CloudInterface):
         blob_client.commit_block_list([], **self._extra_upload_args)
         blob_client.delete_blob()
 
-    def delete_objects(self, paths):
+    def _delete_objects_batch(self, paths):
         """
         Delete the objects at the specified paths
 
         :param List[str] paths:
         """
+        super(AzureCloudInterface, self)._delete_objects_batch(paths)
+
         try:
             # If paths is empty because the files have already been deleted then
             # delete_blobs will return successfully so we just call it with whatever
@@ -485,7 +498,4 @@ class AzureCloudInterface(CloudInterface):
                 )
 
         if errors:
-            raise CloudProviderError(
-                "Error from cloud provider while deleting objects - "
-                "please check the Barman logs"
-            )
+            raise CloudProviderError()
