@@ -28,6 +28,8 @@ import json
 import logging
 import sys
 
+from dateutil import tz
+
 from barman.infofile import BackupInfo
 from barman.utils import (
     BarmanEncoder,
@@ -35,6 +37,7 @@ from barman.utils import (
     human_readable_timedelta,
     pretty_size,
     redact_passwords,
+    timestamp,
 )
 from barman.xlog import diff_lsn
 
@@ -237,7 +240,7 @@ def info(message, *args, **kwargs):
 
 def warning(message, *args, **kwargs):
     """
-    Output a message with severity 'INFO'
+    Output a message with severity 'WARNING'
 
     :key bool log: whether to log the message
     """
@@ -377,7 +380,10 @@ def set_output_writer(new_writer, *args, **kwargs):
 
 
 class ConsoleOutputWriter(object):
+    SERVER_OUTPUT_PREFIX = "Server %s:"
+
     def __init__(self, debug=False, quiet=False):
+
         """
         Default output writer that output everything on console.
 
@@ -578,10 +584,9 @@ class ConsoleOutputWriter(object):
             "Recovery completed (start time: %s, elapsed time: %s)",
             results["recovery_start_time"],
             human_readable_timedelta(
-                datetime.datetime.now() - results["recovery_start_time"]
+                datetime.datetime.now(tz.tzlocal()) - results["recovery_start_time"]
             ),
         )
-        self.info("")
         self.info("Your PostgreSQL server has been successfully prepared for recovery!")
 
     def _record_check(self, server_name, check, status, hint, perfdata):
@@ -624,7 +629,7 @@ class ConsoleOutputWriter(object):
         # If server has configuration errors
         elif disabled:
             display_name += " (WARNING: disabled)"
-        self.info("Server %s:" % display_name)
+        self.info(self.SERVER_OUTPUT_PREFIX % display_name)
         self.active = active
 
     def result_check(self, server_name, check, status, hint=None, perfdata=None):
@@ -833,7 +838,7 @@ class ConsoleOutputWriter(object):
 
         :param str server_name: the server we are start listing
         """
-        self.info("Server %s:", server_name)
+        self.info(self.SERVER_OUTPUT_PREFIX, server_name)
 
     def result_status(self, server_name, status, description, message):
         """
@@ -1048,13 +1053,17 @@ class ConsoleOutputWriter(object):
         else:
             self.info("%s - %s", server_name, description)
 
-    def init_show_server(self, server_name):
+    def init_show_server(self, server_name, description=None):
         """
         Init the show-servers command output method
 
         :param str server_name: the server we are displaying
+        :param str,None description: server description if applicable
         """
-        self.info("Server %s:" % server_name)
+        if description:
+            self.info(self.SERVER_OUTPUT_PREFIX % " ".join((server_name, description)))
+        else:
+            self.info(self.SERVER_OUTPUT_PREFIX % server_name)
 
     def result_show_server(self, server_name, server_info):
         """
@@ -1072,7 +1081,7 @@ class ConsoleOutputWriter(object):
 
         :param str server_name: the server we are displaying
         """
-        self.info("Server %s:" % server_name)
+        self.info(self.SERVER_OUTPUT_PREFIX % server_name)
 
     def result_check_wal_archive(self, server_name):
         """
@@ -1217,14 +1226,14 @@ class JsonOutputWriter(ConsoleOutputWriter):
         self.json_output.update(
             {
                 "recovery_start_time": results["recovery_start_time"].isoformat(" "),
-                "recovery_start_time_timestamp": results[
-                    "recovery_start_time"
-                ].strftime("%s"),
+                "recovery_start_time_timestamp": str(
+                    int(timestamp(results["recovery_start_time"]))
+                ),
                 "recovery_elapsed_time": human_readable_timedelta(
-                    datetime.datetime.now() - results["recovery_start_time"]
+                    datetime.datetime.now(tz.tzlocal()) - results["recovery_start_time"]
                 ),
                 "recovery_elapsed_time_seconds": (
-                    datetime.datetime.now() - results["recovery_start_time"]
+                    datetime.datetime.now(tz.tzlocal()) - results["recovery_start_time"]
                 ).total_seconds(),
             }
         )
@@ -1292,7 +1301,7 @@ class JsonOutputWriter(ConsoleOutputWriter):
         if backup_info.status in BackupInfo.STATUS_COPY_DONE:
             output.update(
                 dict(
-                    end_time_timestamp=backup_info.end_time.strftime("%s"),
+                    end_time_timestamp=str(int(timestamp(backup_info.end_time))),
                     end_time=backup_info.end_time.ctime(),
                     size_bytes=backup_size,
                     wal_size_bytes=wal_size,
@@ -1380,9 +1389,9 @@ class JsonOutputWriter(ConsoleOutputWriter):
                 )
             output["base_backup_information"].update(
                 dict(
-                    begin_time_timestamp=data["begin_time"].strftime("%s"),
+                    begin_time_timestamp=str(int(timestamp(data["begin_time"]))),
                     begin_time=data["begin_time"].isoformat(sep=" "),
-                    end_time_timestamp=data["end_time"].strftime("%s"),
+                    end_time_timestamp=str(int(timestamp(data["end_time"]))),
                     end_time=data["end_time"].isoformat(sep=" "),
                 )
             )
@@ -1680,13 +1689,14 @@ class JsonOutputWriter(ConsoleOutputWriter):
         """
         self.json_output[server_name] = dict(description=description)
 
-    def init_show_server(self, server_name):
+    def init_show_server(self, server_name, description=None):
         """
         Init the show-servers command output method
 
         :param str server_name: the server we are displaying
+        :param str,None description: server description if applicable
         """
-        self.json_output[server_name] = {}
+        self.json_output[server_name] = dict(description=description)
 
     def result_show_server(self, server_name, server_info):
         """
@@ -1699,6 +1709,9 @@ class JsonOutputWriter(ConsoleOutputWriter):
             if not isinstance(message, (int, str, bool, list, dict, type(None))):
                 message = str(message)
 
+            # Prevent null values overriding existing values
+            if message is None and status in self.json_output[server_name]:
+                continue
             self.json_output[server_name][status] = message
 
     def init_check_wal_archive(self, server_name):
