@@ -36,6 +36,7 @@ import sys
 from argparse import ArgumentTypeError
 from abc import ABCMeta, abstractmethod
 from contextlib import contextmanager
+from dateutil import tz
 
 from distutils.version import Version
 
@@ -249,8 +250,37 @@ def total_seconds(timedelta):
     if hasattr(timedelta, "total_seconds"):
         return timedelta.total_seconds()
     else:
-        secs = (timedelta.seconds + timedelta.days * 24 * 3600) * 10 ** 6
-        return (timedelta.microseconds + secs) / 10.0 ** 6
+        secs = (timedelta.seconds + timedelta.days * 24 * 3600) * 10**6
+        return (timedelta.microseconds + secs) / 10.0**6
+
+
+def timestamp(datetime_value):
+    """
+    Compatibility method because datetime.timestamp is not available in Python 2.7.
+
+    :param datetime.datetime datetime_value: A datetime object to be converted
+        into a timestamp.
+    :rtype: float
+    """
+    try:
+        return datetime_value.timestamp()
+    except AttributeError:
+        return total_seconds(
+            datetime_value - datetime.datetime(1970, 1, 1, tzinfo=tz.tzutc())
+        )
+
+
+def range_fun(*args, **kwargs):
+    """
+    Compatibility method required while we still support Python 2.7.
+
+    This can be removed when Python 2.7 support is dropped and calling code can
+    reference `range` directly.
+    """
+    try:
+        return xrange(*args, **kwargs)
+    except NameError:
+        return range(*args, **kwargs)
 
 
 def which(executable, path=None):
@@ -297,31 +327,107 @@ class BarmanEncoder(json.JSONEncoder):
     * binary strings (python 3)
     """
 
+    method_list = [
+        "_to_json",
+        "_datetime_to_str",
+        "_timedelta_to_str",
+        "_decimal_to_float",
+        "binary_to_str",
+        "version_to_str",
+    ]
+
     def default(self, obj):
-        # If the object implements to_json() method use it
-        if hasattr(obj, "to_json"):
-            return obj.to_json()
-        # Serialise date and datetime objects using ctime() method
-        if hasattr(obj, "ctime") and callable(obj.ctime):
-            return obj.ctime()
-        # Serialise timedelta objects using human_readable_timedelta()
-        if isinstance(obj, datetime.timedelta):
-            return human_readable_timedelta(obj)
-        # Serialise Decimal objects using their string representation
-        # WARNING: When deserialized they will be treat as float values
-        # which have a lower precision
-        if isinstance(obj, decimal.Decimal):
-            return float(obj)
-        # Binary strings must be decoded before using them in
-        # an unicode string
-        if hasattr(obj, "decode") and callable(obj.decode):
-            return obj.decode("utf-8", "replace")
-        # Manage (Loose|Strict)Version objects as strings.
-        if isinstance(obj, Version):
-            return str(obj)
+        # Go through all methods until one returns something
+        for method in self.method_list:
+            res = getattr(self, method)(obj)
+            if res is not None:
+                return res
+
         # Let the base class default method raise the TypeError
         return super(BarmanEncoder, self).default(obj)
 
+    @staticmethod
+    def _to_json(obj):
+        """
+        # If the object implements to_json() method use it
+        :param obj:
+        :return: None|str
+        """
+        if hasattr(obj, "to_json"):
+            return obj.to_json()
+
+    @staticmethod
+    def _datetime_to_str(obj):
+        """
+        Serialise date and datetime objects using ctime() method
+        :param obj:
+        :return: None|str
+        """
+        if hasattr(obj, "ctime") and callable(obj.ctime):
+            return obj.ctime()
+
+    @staticmethod
+    def _timedelta_to_str(obj):
+        """
+        Serialise timedelta objects using human_readable_timedelta()
+        :param obj:
+        :return: None|str
+        """
+        if isinstance(obj, datetime.timedelta):
+            return human_readable_timedelta(obj)
+
+    @staticmethod
+    def _decimal_to_float(obj):
+        """
+        Serialise Decimal objects using their string representation
+        WARNING: When deserialized they will be treat as float values which have a lower precision
+        :param obj:
+        :return: None|float
+        """
+        if isinstance(obj, decimal.Decimal):
+            return float(obj)
+
+    @staticmethod
+    def binary_to_str(obj):
+        """
+        Binary strings must be decoded before using them in an unicode string
+        :param obj:
+        :return: None|str
+        """
+        if hasattr(obj, "decode") and callable(obj.decode):
+            return obj.decode("utf-8", "replace")
+
+    @staticmethod
+    def version_to_str(obj):
+        """
+        Manage (Loose|Strict)Version objects as strings.
+        :param obj:
+        :return: None|str
+        """
+        if isinstance(obj, Version):
+            return str(obj)
+
+
+class BarmanEncoderV2(BarmanEncoder):
+    """
+    This class purpose is to replace default datetime encoding from ctime to isoformat (ISO 8601).
+    Next major barman version will use this new format. So this class will be merged back to BarmanEncoder.
+    """
+
+    @staticmethod
+    def _datetime_to_str(obj):
+        """
+        Try set output isoformat for this datetime. Date must have tzinfo set.
+        :param obj:
+        :return: None|str
+        """
+        if isinstance(obj, datetime.datetime):
+            if obj.tzinfo is None:
+                raise ValueError(
+                    'Got naive datetime. Expecting tzinfo for date: "{}"'.format(obj)
+                )
+            return obj.isoformat()
+
 
 def fsync_dir(dir_path):
     """
@@ -608,12 +714,26 @@ def check_positive(value):
     try:
         int_value = int(value)
     except Exception:
-        raise ArgumentTypeError("'%s' is not a valid positive integer" % value)
+        raise ArgumentTypeError("'%s' is not a valid input" % value)
     if int_value < 1:
         raise ArgumentTypeError("'%s' is not a valid positive integer" % value)
     return int_value
 
 
+def check_tli(value):
+    """
+    Check for a positive integer option, and also make "current" and "latest" acceptable values
+
+    :param value: str containing the value to check
+    """
+    if value is None:
+        return None
+    if value in ["current", "latest"]:
+        return value
+    else:
+        return check_positive(value)
+
+
 def check_size(value):
     """
     Check user input for a human readable size
